/*********************************************
 *                                           *
 *         Programa Editor em Janela         *
 *         Modulo de edicao.                 *
 *         Usa ncurses(3)                    *
 *                                           *
 *********************************************/

#include <curses.h>
#include "edicao.h"

/* Linha e coluna correntes */
static int curline = 0,
           curcol = 0;

/* Indice de janela corrente */
static int  icurwin;

/* Areas de edicao */
static WINDOW *ed[3],  /* Indice JANELA_MSGS contem janela de Mensagens */
              *curwin; /* Area de edicao atual */


/* Desenha linha vertical pela tela em uma coluna especificada */
static void dvline (int col)
{
  int i;

  for (i = 0; i < LINES - 1; ++i)
    mvaddch(i, col, DIV);

  refresh();
}


/* Desenha linha horizontal pela tela em uma linha especificada */
static void dhline (int lin)
{
  int i;

  for (i = 0; i < COLS; ++i)
    mvaddch(lin, i, HIFEN);

  refresh();
}


/* Le um char do teclado  e insere na janela. Manipula mudanca de linha
 * e final de tela */
static add(WINDOW *win)
{
  int in;

  /* Le caracteres ateh ESCAPE */
  while ((in = getch()) != ESC) {
    /* Ecoa caracter e obtem nova pos do cursor de texto */
    waddch(win, in);
    getyx(win, curline, curcol);

    /* Se teclou  RETURN e nao estah no final da  tela, cursor vai para
     * proxima linha */
    if (in == ENTER && curline != LINES - 2)
      ++curline;

    wmove(win, curline, curcol);
    wrefresh(win);
  }
}


/* Define 2 areas de edicao do mesmo tamanho e a area de mensagens */
void setWindows (int numWindows)
{
  int i;

  for (i=0; i < numWindows; ++i) {
    ed[i] = newwin (LINES - 2, COLS / 2 - 1, 0, i * (COLS / 2 + 1));

    if (i) {
      /* Desenha linhas entre as areas de edicao */
      dvline (i * (COLS / 2 - 1));
      dvline (i * COLS / 2 );
    }
  }

  /* Define a area de mensagens */
  ed[JANELA_MSGS] = newwin (2, COLS - 1, LINES - 2, 0);
  dhline (LINES - 2);
  
}


/* Devolve o objeto WINDOW correspondente a um indice de janela */
WINDOW *getWindow (int winidx)
{
  return (ed[winidx]);  
}

/* Imprime  mensagem ao final da janela corrente */
void printMsg(char *mensagem)
{
    mvwprintw (ed[JANELA_MSGS], 1, 1, mensagem);
    wrefresh (ed[JANELA_MSGS]);
    dhline (LINES - 2);
    wrefresh (ed[JANELA_MSGS]);
}


/* Inicia edicao na Janela indicada */
initEdit (int id_janela)
{
  icurwin = id_janela;
  curwin = ed[icurwin];

}


/* Processa comandos de edicao */
void processar ()
{
  int in;

  for (;;) {
    /* Faz refresh da tela */
    wmove(curwin, curline, curcol);
    wrefresh (curwin);
    
    /* Obtem comando do teclado e o processa */
    in = getch();

    switch (in) {
    case 's':           /* Troca cursor de janela */
      curwin = ed[1 - icurwin];
      icurwin = 1 - icurwin;
      getyx (curwin, curline, curcol);
      break;
    case 'j':           /* Movimenta cursor uma linha abaixo */
    case KEY_DOWN:
      if (curline != LINES - 2)
	++curline;
      break;
    case 'k':           /* Movimenta cursor uma linha acima */
    case KEY_UP:
      if (curline != 0)
	--curline;
      break;
    case 'h':           /* Movimenta cursor um caracter a esquerda */
    case KEY_LEFT:
      if (curcol != 0)
	--curcol;
      break;
    case 'l':           /* Movimenta cursor um caracter a direita */
    case KEY_RIGHT:
      if (curcol != COLS / 2 - 2)
	++curcol;
      break;
    case 'd':           /* Remove linha onde estah cursor */
      wdeleteln (curwin);
      break;
    case 'i':           /* Modo insercao a partir do cursor */
      add (curwin);
      break;
    case 'o':           /* Abre uma linha abaixo da linha corrente */
      /* move cursor uma linha para baixo se nao estiver na ultima linha */
      if (curline != LINES - 2)
	++curline;
      else    /* ignore comando 'o' se na ultima linha */
	break;

      curcol = 0;
      wmove(curwin, curline, curcol);
      winsertln (curwin);
      wrefresh (curwin);
      add (curwin);
      break;
    case 'O':           /* Abre uma linha acima da linha corrente */
      winsertln (curwin);
      curcol = 0;
      wmove(curwin, curline, curcol);
      wrefresh (curwin);
      add (curwin);
      break;
    case 'x':           /* Apaga caracter sob cursor */
      wdelch (curwin);
      break;
    case 'D':           /* Apaga caracteres do cursor ao final da linha */
      wclrtoeol(curwin);
      break;
    case 'w':           /* Salva arquivo e termina programa */
      saveFiles();
      return ;
    case 'q':           /* Termina programa sem salvar arquivo */
      return ;
    }
  }
}

